%The sample matlab code is for semi-supervised NMF on toy data
%Copyright (C) 2007 by Yanhua Chen (Email: chenyanh@wayne.edu)
%If you find the algorithm or code is useful, we appreciate it very much if you can cite our following works:
% (1)Yanhua Chen, Manjeet Rege, Ming Dong and Jing Hua, 
% "Non-negative Matrix Factorization for Semi-supervised Data Clustering", Journal of Knowledge and Information Systems (Springer),
% Vol. 17, No. 3, pp. 355 - 379, 2008. (invited as a best paper of ICDM 07, 
% conference version: "Incorporating User provided Constraints into Document Clustering", 
% was published in Proc. of IEEE International Conference on Data Mining, Omaha, NE, pp. 103-112, 2007 as a Regular paper, acceptance rate 7.2%). 
% (2) Yanhua Chen, Manjeet Rege, Ming Dong and Farshad Fotouhi, "Deriving Semantics for Image Clustering from Accumulated User Feedbacks", 
% Proc. of ACM Multimedia, Germany, pp. 313-316, 2007 (acceptance rate: 24%). 


clear all;
close all;

X = [0.9157       0
    0.7228    0.6065
    0.1732    0.9820
   -0.4378    0.7583
   -1.0086    0.3671
   -0.9453   -0.3441
   -0.5075   -0.8789
    0.2014   -1.1420
    0.7065   -0.5928
    1.1550   -0.0000
    6.1055    3.0000
    5.2854    4.9177
    3.5264    5.9854
    1.4290    5.7210
    0.1502    4.0373
    0.1362    1.9577
    1.4801    0.3674
    3.5197    0.0527
    5.3989    0.9871
    6.0978    3.0000];

N1=10;
N2=10;

t_size=N1+N2;

figure(1);
plot(X(1:10,1),X(1:10, 2),'o', X(11:20, 1), X(11:20, 2),'*');

true_label_ini1=ones(N1,1);
true_label_ini2=ones(N2,1)*2;
true_label =[true_label_ini1; true_label_ini2];
gnd=true_label;


V=X*X'; %similarity matrix
Z=V-min(min(V)); %enforce non-negative matrix

min_V=min(min(V));
max_V=max(max(V));

V_ori=V;

min_V=min(min(V_ori));
max_V=max(max(V_ori));

V=V-min_V;


[vrow, vcol]=size(V);

Vc=zeros(vrow,vcol);

%add constraits
con_num=round(t_size*(t_size-1)/2*0.3)
iteration=con_num;

for itera = 1:iteration 
   k_rand = round(t_size*t_size*rand); 
   i =rem(k_rand, t_size)+1
   k_rand = round(t_size*t_size*rand);
   j =rem(k_rand, t_size)+1
  if V_ori(i,j)~=min_V && V_ori(i,j)~=max_V 
      if true_label(i)==true_label(j) 
          V_ori(i,j) = max_V;
          V_ori(j,i) = max_V; 
      else
          V_ori(i,j) = min_V;
          V_ori(j,i) = min_V;       
      end
  end
end
Vc=Vc + V_ori;

Zc=Vc-min(min(Vc));
[vrow, vcol]=size(Z);

% Check that we have non-negative data
if min(Z(:))<0, error('Negative values in data!'); end

% Globally rescale data to avoid potential overflow/underflow
Z = Z/max(Z(:));

%k is number of clusters
k=2;


% Create initial matrices, will affect the final result, please try some
% times
G_ini= abs(randn(vrow,k));
S_ini= abs(randn(k,k));

G=G_ini;
S=S_ini;

% Calculate initial objective
objhistory = 0.5*sum(sum((Z-G*S*G').^2));

timestarted = clock;

% Start iteration
iter = 0;
while 1,
    % Save time
    if rem(iter,5)==0,
	elapsed = etime(clock,timestarted);
    end
	
    %plot the convergence of objective function
	if iter>1,
	    figure(2);
	    plot(objhistory(2:end));
	end
	drawnow;
   
    
    % Update iteration count
    iter = iter+1;    
    
    % Save old values
    Gold = G;
    Sold = S;
    
    % Compute new G and S 
    G = G.*(Z*G*S)./(G*S*G'*G*S+ 1e-9);
    S = S.*(G'*Z*G)./(G'*G*S*G'*G+ 1e-9);
  
     % Calculate objective
    newobj = 0.5*sum(sum((Z-G*S*G').^2));
    objhistory = [objhistory newobj];
    
    
   if iter > 100 %note that this iteration number can be adjusted
        break;
   end
   
end

%normalize G and S
norms = sqrt(sum(G.^2))+1e-9;
%G = G./(norms'*ones(1,vrow));
G = G./(ones(vrow,1)*norms); %cluster indicator
S= diag(norms); %cluster centroid

 GSG = G*S*G'; 
 Emse = 0.5*sum(sum((Z-GSG).^2));
  
  
%   fprintf('---------------------------------\n');
%   fprintf('Approximation error:\n');
%   fprintf('MSE = %.5f \n',Emse);
%  
%   % SHOW HOW MUCH TIME IT TOOK...
%   
%   fprintf('Running time (in seconds):\n');
%   fprintf('%.1f\n',elapsed);
%   fprintf('---------------------------------\n');

  
[Gmax, I1]=max(G, [], 2);
res1=I1;

res1 = bestMap(gnd,res1);
%============= evaluate AC: accuracy without constraits ==============
AC_without_constr = length(find(gnd == res1))/length(gnd)
%============= evaluate MIhat: nomalized mutual information without constraits=================
MIhat_without_constr = MutualInfo(gnd,res1)

[vrow, vcol]=size(Zc);

% Check that we have non-negative data
if min(Zc(:))<0, error('Negative values in data!'); end

% Globally rescale data to avoid potential overflow/underflow
Zc = Zc/max(Zc(:));

%k is number of clusters
k=2;

Gc=G_ini;
Sc=S_ini;

% Calculate initial objective
objhistory = 0.5*sum(sum((Zc-Gc*Sc*Gc').^2));

timestarted = clock;

% Start iteration
iter = 0;
while 1,
   
    if rem(iter,5)==0,
	elapsedc = etime(clock,timestarted);
    end
	
   
	if iter>1,
	    figure(3);
	    plot(objhistory(2:end));
	end
	drawnow;
    
    % Update iteration count
    iter = iter+1;    
    
    % Save old values
    Gcold = Gc;
    Scold = Sc;
 
    
    % Compute new Gc and Sc
    Gc = Gc.*(Zc*Gc*Sc)./(Gc*Sc*Gc'*Gc*Sc+ 1e-9);
    Sc = Sc.*(Gc'*Zc*Gc)./(Gc'*Gc*Sc*Gc'*Gc+ 1e-9);
   
 
     % Calculate objective
    newobj = 0.5*sum(sum((Zc-Gc*Sc*Gc').^2));
    objhistory = [objhistory newobj];
    
    
   if iter > 100
        break;
    end
   
end

%normalize Gc and Sc
normsc = sqrt(sum(Gc.^2))+1e-9;
Gc = Gc./(ones(vrow,1)*normsc); %new cluster indicator
Sc= diag(normsc); %new cluster centroid

 GSGc = Gc*Sc*Gc'; 
 Emsec = 0.5*sum(sum((Zc-GSGc).^2));
  
  
%   fprintf('---------------------------------\n');
%   fprintf('Approximation error:\n');
%   fprintf('MSE = %.5f \n',Emsec);
%  
%   % SHOW HOW MUCH TIME IT TOOK...
%   
%   fprintf('Running time (in seconds):\n');
%   fprintf('%.1f\n',elapsedc);
%   fprintf('---------------------------------\n');

 
[Gcmax, I1c]=max(Gc, [], 2);
res2=I1c;

res2 = bestMap(gnd,res2);
%============= evaluate AC: accuracy after adding constraits==============
AC_add_constr = length(find(gnd == res2))/length(gnd)
%============= evaluate MIhat: nomalized mutual information after adding constraits=================
MIhat_add_constr = MutualInfo(gnd,res2)
